@@ -392,6 +392,18 @@ seafile_get_repo_sync_task (const char *repo_id, GError **error)
         return (GObject *)s_task;
     }
 
+    if (repo->empty_enc_key) {
+        SeafileSyncTask *s_task;
+        s_task = g_object_new (SEAFILE_TYPE_SYNC_TASK,
+                               "force_upload", FALSE,
+                               "state", "error",
+                               "error", SYNC_ERROR_ID_CORRUPTED_ENC_KEY,
+                               "repo_id", repo_id,
+                               NULL);
+
+        return (GObject *)s_task;
+    }
+
     SyncInfo *info = seaf_sync_manager_get_sync_info (seaf->sync_mgr, repo_id);
     if (!info || !info->current_task)
         return NULL;
@@ -780,9 +792,9 @@ seafile_unsync_repos_by_account (const char *server_url, const char *email, GErr
     }
 
     g_list_free (repos);
-    g_free (canon_server_url);
 
-    cancel_clone_tasks_by_account (server_url, email);
+    cancel_clone_tasks_by_account (canon_server_url, email);
+    g_free (canon_server_url);
 
     return 0;
 }
@@ -811,9 +823,9 @@ seafile_remove_repo_tokens_by_account (const char *server_url, const char *email
     }
 
     g_list_free (repos);
-    g_free (canon_server_url);
 
-    cancel_clone_tasks_by_account (server_url, email);
+    cancel_clone_tasks_by_account (canon_server_url, email);
+    g_free (canon_server_url);
 
     return 0;
 }
