@@ -255,7 +255,7 @@ register_rpc_service ()
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_generate_magic_and_random_key,
                                      "seafile_generate_magic_and_random_key",
-                                     searpc_signature_object__int_string_string());
+                                     searpc_signature_object__int_string_string_string_string());
 
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_get_server_property,
@@ -534,6 +534,8 @@ main (int argc, char **argv)
 
     seafile_event_message("Starting record seafile events.\n");
 
+    seafile_log_rotate_start ();
+
     /* init seafile */
     if (seafile_dir == NULL)
         seafile_dir = g_build_filename (config_dir, "seafile-data", NULL);
